@@ -60,6 +60,9 @@ on_playlist_treeview_button_release (GtkWidget      *widget,
                                      GdkEventButton *event,
                                      gpointer        user_data);
 void
+on_playlist_treeview_item_selected (GtkTreeSelection *selection,
+                                    gpointer          user_data);
+void
 on_playlist_row_expanded (GtkTreeView *tree_view,
                           GtkTreeIter *iter,
                           GtkTreePath *path,
@@ -75,6 +78,9 @@ void
 on_didl_menuitem_activate (GtkMenuItem *menuitem,
                            gpointer     user_data);
 
+gboolean
+on_playlist_treeview_popup_menu (GtkWidget *widget, gpointer user_data);
+
 static void
 on_proxy_ready (GObject *source_object,
                 GAsyncResult *res,
@@ -147,6 +153,25 @@ browse (AVCPMediaServer *content_dir,
         guint32          starting_index,
         guint32          requested_count);
 
+static void
+do_popup_menu (GtkMenu *menu, GtkWidget *widget, GdkEventButton *event)
+{
+#if GTK_CHECK_VERSION(3,22,0)
+        gtk_menu_popup_at_pointer (menu, (event != NULL) ? (GdkEvent *) event : gtk_get_current_event ());
+#else
+        int button = 0;
+        int event_time;
+        if (event) {
+                button = event->button;
+                event_time = event->time;
+        } else {
+                event_time = gtk_get_current_event_time ();
+        }
+
+        gtk_menu_popup (menu, NULL, NULL, NULL, NULL, button, event_time);
+#endif
+}
+
 G_MODULE_EXPORT
 gboolean
 on_playlist_treeview_button_release (GtkWidget      *widget,
@@ -168,20 +193,24 @@ on_playlist_treeview_button_release (GtkWidget      *widget,
         if (!gtk_tree_selection_get_selected (selection, &model, &iter))
                 return FALSE;
 
-        gtk_menu_popup (GTK_MENU (popup),
-                        NULL,
-                        NULL,
-                        NULL,
-                        NULL,
-                        event->button,
-                        event->time);
+        do_popup_menu (GTK_MENU (popup), treeview, event);
 
         return TRUE;
 }
 
-static void
-on_item_selected (GtkTreeSelection *selection,
-                  gpointer          user_data)
+G_MODULE_EXPORT
+gboolean
+on_playlist_treeview_popup_menu (GtkWidget *widget, gpointer user_data)
+{
+        do_popup_menu (GTK_MENU (popup), treeview, NULL);
+
+        return TRUE;
+}
+
+G_MODULE_EXPORT
+void
+on_playlist_treeview_item_selected (GtkTreeSelection *selection,
+                           gpointer          user_data)
 {
         PlaybackState state;
 
@@ -196,65 +225,6 @@ on_item_selected (GtkTreeSelection *selection,
        }
 }
 
-static GtkTreeModel *
-create_playlist_treemodel (void)
-{
-        GtkTreeStore *store;
-
-        store = gtk_tree_store_new (7,
-                                    /* Icon */
-                                    GDK_TYPE_PIXBUF,
-                                    /* Title */
-                                    G_TYPE_STRING,
-                                    /* MediaServer */
-                                    GUPNP_TYPE_DEVICE_PROXY,
-                                    /* Content Directory */
-                                    GUPNP_TYPE_SERVICE_PROXY,
-                                    /* Id */
-                                    G_TYPE_STRING,
-                                    /* Is container? */
-                                    G_TYPE_BOOLEAN,
-                                    /* childCount */
-                                    G_TYPE_INT);
-
-        return GTK_TREE_MODEL (store);
-}
-
-static void
-setup_treeview_columns (GtkWidget *tv)
-
-{
-        GtkTreeSelection  *selection;
-        GtkTreeViewColumn *column;
-        GtkCellRenderer   *renderers[2];
-        const char        *headers[] = { "Icon", "Title"};
-        const char        *attribs[] = { "pixbuf", "text"};
-        int                i;
-
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-        g_assert (selection != NULL);
-
-        column = gtk_tree_view_column_new ();
-
-        renderers[0] = gtk_cell_renderer_pixbuf_new ();
-        renderers[1] = gtk_cell_renderer_text_new ();
-
-        for (i = 0; i < 2; i++) {
-                gtk_tree_view_column_pack_start (column, renderers[i], FALSE);
-                gtk_tree_view_column_set_title (column, headers[i]);
-                gtk_tree_view_column_add_attribute (column,
-                                                    renderers[i],
-                                                    attribs[i], i);
-                gtk_tree_view_column_set_sizing(column,
-                                                GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-        }
-
-        gtk_tree_view_insert_column (GTK_TREE_VIEW (treeview),
-                                     column,
-                                     -1);
-
-        gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-}
 
 G_MODULE_EXPORT
 void
@@ -361,9 +331,6 @@ on_didl_menuitem_activate (GtkMenuItem *menuitem,
 void
 setup_playlist_treeview (GtkBuilder *builder)
 {
-        GtkTreeModel      *model;
-        GtkTreeSelection  *selection;
-
         initial_notify = g_hash_table_new (g_direct_hash, g_direct_equal);
 
         treeview = GTK_WIDGET (gtk_builder_get_object (builder,
@@ -403,21 +370,6 @@ setup_playlist_treeview (GtkBuilder *builder)
         }
 #endif
 
-        model = create_playlist_treemodel ();
-        g_assert (model != NULL);
-
-        gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
-        g_object_unref (model);
-
-        setup_treeview_columns (treeview);
-
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-        g_assert (selection != NULL);
-        g_signal_connect (selection,
-                          "changed",
-                          G_CALLBACK (on_item_selected),
-                          NULL);
-
         expanded = FALSE;
 }
 
@@ -436,20 +388,12 @@ compare_media_server (GtkTreeModel *model,
         gtk_tree_model_get (model, iter,
                             2, &info, -1);
 
-        if (info) {
-                if (GUPNP_IS_DEVICE_PROXY (info)) {
-                        const char *device_udn;
-
-                        device_udn = gupnp_device_info_get_udn (info);
-
-                        if (device_udn && strcmp (device_udn, udn) == 0) {
-                                found = TRUE;
-                        }
-                }
-
-                g_object_unref (info);
+        if (info && GUPNP_IS_DEVICE_PROXY (info)) {
+                found = g_strcmp0 (gupnp_device_info_get_udn (info), udn) == 0;
         }
 
+        g_clear_object (&info);
+
         return found;
 }
 
@@ -518,14 +462,11 @@ compare_container (GtkTreeModel *model,
                 return found;
         }
 
-        if (container_id == NULL)
-                return found;
-
-        if (strcmp (container_id, id) == 0) {
+        if (g_strcmp0 (container_id, id) == 0) {
                 found = TRUE;
         }
 
-        g_free (container_id);
+        g_clear_pointer (&container_id, g_free);
 
         return found;
 }
